﻿using iFileProxy.Config;
using Serilog;
using System.Data;
using iFileProxy.Helpers;
using MySql.Data.MySqlClient;
using iFileProxy.Models;
using Newtonsoft.Json;

namespace iFileProxy.Services
{
    /// <summary>
    /// 数据库访问网关服务
    /// 提供统一的数据库操作接口，管理数据库连接和查询
    /// </summary>
    public class DatabaseGateService
    {
        Database _db;
        AppConfig _appConfig;
        private readonly Serilog.ILogger _logger = Log.Logger.ForContext<DatabaseGateService>();

        Dictionary<string, DB> _dbDictionary = [];

        /// <summary>
        /// 初始化数据库网关服务
        /// </summary>
        /// <param name="appConfig">应用程序配置</param>
        public DatabaseGateService(AppConfig appConfig)
        {
            _logger.Information("Initializing DatabaseGateService...");
            _db = appConfig.Database;
            _appConfig = appConfig;
            try
            {
                _logger.Information("Done.");
            }
            catch (Exception e)
            {
                _logger.Fatal($"程序异常: {e.Message}");
            }
            LoadDbDict();
        }

        /// <summary>
        /// 加载数据库配置字典
        /// </summary>
        public void LoadDbDict()
        {
            foreach (DB item in _db.Databases)
            {
                _dbDictionary.Add(item.Description, item);
                _logger.Debug($"Db Config: {item.Description} <=> {item.DatabaseName} Loaded.");
            }
        }

        /// <summary>
        /// 获取并打开指定数据库的连接
        /// </summary>
        /// <param name="db_desc">数据库描述符，对应配置文件中的 Description 字段</param>
        /// <returns>已打开的数据库连接</returns>
        /// <exception cref="Exception">当找不到匹配的数据库配置时抛出</exception>
        /// <exception cref="NoNullAllowedException">当必需的配置字段为空时抛出</exception>
        public MySqlConnection GetAndOpenDBConn(string db_desc)
        {
            if (!_dbDictionary.TryGetValue(db_desc, out DB Db))
            {
                throw new Exception($"未找到与 {db_desc} 相匹配的数据库配置");
            }

            var db_user = Db.User ?? _db.Common.User;
            var db_password = Db.Password ?? _db.Common.Password;
            var db_host = Db.Host ?? _db.Common.Host;
            var db_port = Db.Port ?? _db.Common.Port;

            if (db_user == null || db_password == null || db_host == null || db_port == null)
                throw new NoNullAllowedException("数据库配置获取失败,不允许为空的字段出现空值");

            string db_connstr = $"server={db_host};user={db_user};database={Db.DatabaseName};port={db_port};password={db_password};Pooling=true;MaximumPoolSize=500;";
            MySqlConnection conn;
            try
            {
                conn = new MySqlConnection(db_connstr);
                conn.Open();
            }
            catch (Exception ex)
            {
                _logger.Fatal($"获取Mysql连接时出现异常:{ex.Message}");
                throw;
            }
            return conn;
        }

        /// <summary>
        /// 测试所有数据库配置的连接
        /// </summary>
        /// <returns>测试是否全部成功</returns>
        public bool TestDbConfig()
        {
            foreach (var db in _dbDictionary)
            {
                _logger.Information($"[程序启动前配置验证] 正在测试数据库配置: {db.Key} ...");
                MySqlConnection dbConn = new();
                try
                {
                    dbConn = GetAndOpenDBConn(db.Key);
                    _logger.Information($"succ.");
                }
                catch (Exception)
                {
                    _logger.Fatal($"=========== 数据库: {db.Key} 测试失败! ===========");
                    return false;
                }
                finally
                {
                    dbConn.Close();
                }
            }
            return true;
        }

        /// <summary>
        /// 执行查询并返回 JSON 格式的结果
        /// </summary>
        /// <param name="sqlCmd">SQL 命令对象</param>
        /// <returns>JSON 格式的查询结果</returns>
        public static string QueryTableData(MySqlCommand sqlCmd)
        {
            DataTable dataTable = new();
                using (MySqlDataAdapter adapter = new (sqlCmd))
                    adapter.Fill(dataTable);
            return JsonConvert.SerializeObject(dataTable);
        }

        /// <summary>
        /// 执行查询并返回 JSON 格式的结果
        /// </summary>
        /// <param name="sql">SQL 查询语句</param>
        /// <param name="dbConfName">数据库配置名称</param>
        /// <returns>JSON 格式的查询结果</returns>
        public string QueryTableData(string sql, string dbConfName)
        {
            DataTable dataTable = new();
            using (MySqlDataAdapter adapter = new(new MySqlCommand(sql, GetAndOpenDBConn(dbConfName))))
                adapter.Fill(dataTable);
            return JsonConvert.SerializeObject(dataTable);
        }

        /// <summary>
        /// 执行非查询SQL语句
        /// </summary>
        /// <param name="sql">SQL语句</param>
        /// <param name="dbConfName">数据库配置名称</param>
        /// <returns>受影响的行数</returns>
        /// <remarks>
        /// 警告：此方法仅供内部使用，直接暴露给客户端可能导致SQL注入风险
        /// </remarks>
        public int Query(string sql, string dbConfName)
        {
            using MySqlCommand sqlCmd = new (sql, GetAndOpenDBConn(dbConfName));
            int n = sqlCmd.ExecuteNonQuery();
            _logger.Debug($"查询完成, 受影响的行数: {n}");
            return n;
        }

        /// <summary>
        /// 检查缓存依赖关系
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <param name="ipAddr">IP地址</param>
        /// <returns>相关的任务信息列表</returns>
        public List<TaskInfo> CheckCacheDependencies(string taskId, string ipAddr)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE `status` = @status AND `tag` = @tag AND `client_ip` <> @ip_addr";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tag", $"REDIRECT:{taskId}");
                sqlCmd.Parameters.AddWithValue("@status", TaskState.Cached);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);

                return JsonConvert.DeserializeObject<List<TaskInfo>>(QueryTableData(sqlCmd));
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        /// <summary>
        /// 根据状态和IP地址获取任务列表
        /// </summary>
        /// <param name="ipAddr">IP地址</param>
        /// <param name="status">任务状态</param>
        /// <returns>JSON格式的任务列表</returns>
        public string GetTaskListByStateAndIp(string ipAddr, TaskState status)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE client_ip = @ip_addr AND `status` = @status";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);
                sqlCmd.Parameters.AddWithValue("@status", status);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        /// <summary>
        /// 根据IP地址获取任务列表
        /// </summary>
        /// <param name="ipAddr">IP地址</param>
        /// <returns>JSON格式的任务列表</returns>
        public string GetTaskListByIP(string ipAddr)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE client_ip = @ip_addr";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new (sql,conn);
                sqlCmd.Parameters.AddWithValue("@ip_addr", ipAddr);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务列表: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        /// <summary>
        /// 根据任务ID获取任务信息
        /// </summary>
        /// <param name="tid">任务ID</param>
        /// <returns>JSON格式的任务信息</returns>
        public string GetTaskInfoByTid(string tid)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE `tid` =@tid";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tid", tid);
                return QueryTableData(sqlCmd);
            }
            catch (Exception e)
            {
                _logger.Error($"无法获取任务信息: {e.Message}");
                throw;
            }
            finally { conn.Close(); }
        }

        /// <summary>
        /// 查询任务信息
        /// </summary>
        /// <param name="fileName">文件名</param>
        /// <param name="url">URL</param>
        /// <param name="size">文件大小</param>
        /// <param name="status">任务状态</param>
        /// <returns>任务信息对象，如果未找到则返回null</returns>
        public TaskInfo? QueryTaskInfo(string fileName, string url, long size, TaskState status)
        {
            string sql = $"SELECT * FROM t_tasks_info WHERE url = @url AND size = @size AND `status` = @status AND file_name = @fileName";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try {
                MySqlCommand sqlCmd = new MySqlCommand(sql, conn);
                sqlCmd.Parameters.AddWithValue("@url", url);
                sqlCmd.Parameters.AddWithValue("@size", size);
                sqlCmd.Parameters.AddWithValue("@status", status);
                sqlCmd.Parameters.AddWithValue("@fileName", fileName);
                string result = QueryTableData(sqlCmd);
                List<TaskInfo>? r = JsonConvert.DeserializeObject<List<TaskInfo>>(result);
                if (r != null)
                {
                    return r[0];
                }
                else
                    return null;
            }
            catch (Exception e){
                return null;
            }
            finally { 
                conn.Close();
            }

        }

        /// <summary>
        /// 插入任务数据
        /// </summary>
        /// <param name="taskInfo">任务信息对象</param>
        /// <returns>是否插入成功</returns>
        public bool InsertTaskData(TaskInfo taskInfo)
        {
            _logger.Debug(System.Text.Json.JsonSerializer.Serialize(taskInfo));
            string sql = "INSERT INTO `t_tasks_info` (`tid`, `file_name`, `client_ip`, `add_time`, `update_time`, `status`, `url`, `size`, `hash`, `tag`) " +
                "VALUES (@tid, @file_name, @client_ip, @add_time, @update_time, @status, @url, @size, @hash, @tag)";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");

            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@tid", taskInfo.TaskId);
                sqlCmd.Parameters.AddWithValue("@file_name", taskInfo.FileName);
                sqlCmd.Parameters.AddWithValue("@client_ip", taskInfo.ClientIp);
                sqlCmd.Parameters.AddWithValue("@add_time", taskInfo.AddTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@update_time", taskInfo.UpdateTime.ToString("yyyy-MM-dd HH:mm:ss"));
                sqlCmd.Parameters.AddWithValue("@status", taskInfo.Status);
                sqlCmd.Parameters.AddWithValue("@url", taskInfo.Url);
                sqlCmd.Parameters.AddWithValue("@size", taskInfo.Size);
                sqlCmd.Parameters.AddWithValue("@hash", taskInfo.Hash);
                sqlCmd.Parameters.AddWithValue("@tag", taskInfo.Tag);

                sqlCmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
                _logger.Fatal($"插入数据时出现问题");
                throw;
            }
            finally
            {
                conn.Close();
            }
            return true;
        }

        /// <summary>
        /// 更新指定字段的数据
        /// </summary>
        /// <param name="fieldsName">字段名</param>
        /// <param name="key">主键值</param>
        /// <param name="val">更新值</param>
        /// <returns>是否更新成功</returns>
        public bool UpdateFieldsData(string fieldsName, string key, string val)
        {
            string sql = $"UPDATE t_tasks_info set `{fieldsName}` = @Data WHERE `tid` = @tid";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new(sql, conn);
                sqlCmd.Parameters.AddWithValue("@Data",val);
                sqlCmd.Parameters.AddWithValue("@tid",key);
                if (sqlCmd.ExecuteNonQuery() == 1)
                {
                    return true;
                }
                else
                return false;
            }
            catch (Exception)
            {
                _logger.Fatal($"Task Data update error.");
                throw;
            }
            finally
            {
                conn.Close();
            }

        }

        /// <summary>
        /// 更新任务状态
        /// </summary>
        /// <param name="taskInfo">任务信息对象</param>
        /// <returns>是否更新成功</returns>
        public bool UpdateTaskStatus(TaskInfo taskInfo)
        {
            string sql = @"UPDATE t_tasks_info set `status` = @status , update_time = Now() WHERE `tid` = @tid";
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");
            try
            {
                using MySqlCommand sqlCmd = new (sql, conn);
                sqlCmd.Parameters.AddWithValue("@status", taskInfo.Status);
                sqlCmd.Parameters.AddWithValue("@tid", taskInfo.TaskId);
                if (sqlCmd.ExecuteNonQuery() >= 1)
                { 
                    _logger.Debug($"Task: {taskInfo.TaskId} Status Change to {taskInfo.Status}"); 
                    return true;
                }
                else
                    _logger.Warning($"Task: {taskInfo.TaskId} Status Change Failed.");
                return false;
            }
            catch (Exception)
            {
                _logger.Fatal($"Task status update error.");
                throw;
            }
            finally
            { 
                conn.Close ();
            }
        }

        /// <summary>
        /// 更新任务的哈希值
        /// </summary>
        /// <param name="taskInfo">任务信息对象</param>
        /// <returns>是否更新成功</returns>
        public bool UpdateTaskHash(TaskInfo taskInfo)
        {
            return UpdateFieldsData("hash", taskInfo.TaskId, MasterHelper.GetFileHash(Path.Combine(_appConfig.DownloadOptions.SavePath, taskInfo.FileName), FileHashAlgorithm.MD5));
        }

        /// <summary>
        /// 删除指定IP地址的所有任务信息
        /// </summary>
        /// <param name="ipAddr">IP地址</param>
        /// <returns>受影响的行数，-1表示操作失败</returns>
        public int DeleteTaskInfoByIpAddr(string ipAddr)
        { 
            try
            {
                string sql = $"DELETE FROM `t_tasks_info` WHERE `client_ip` = '{ipAddr}'";
                return Query(sql,DbConfigName.iFileProxy);
            }
            catch (Exception)
            {
                return -1;
            }
        }

        /// <summary>
        /// 尝试初始化数据库
        /// </summary>
        /// <remarks>
        /// 创建必要的数据库表结构
        /// </remarks>
        public void TryInitialDB()
        {
            string sql = """
                   CREATE TABLE `t_tasks_info`  (
                      `id` int(11) NOT NULL AUTO_INCREMENT COMMENT '自增键',
                      `tid` varchar(48) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL COMMENT '任务id',
                      `file_name` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NOT NULL,
                      `client_ip` varchar(15) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '客户端IP',
                      `add_time` datetime NOT NULL COMMENT '任务在何时被添加',
                      `update_time` datetime NOT NULL COMMENT '任务状态更新时间',
                      `status` int(11) NULL DEFAULT NULL COMMENT '任务状态',
                      `url` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '文件url',
                      `size` int(11) NULL DEFAULT NULL COMMENT '文件大小',
                      `hash` varchar(255) CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci NULL DEFAULT NULL COMMENT '文件hash',
                      PRIMARY KEY (`id`) USING BTREE
                ) ENGINE = InnoDB AUTO_INCREMENT = 17 CHARACTER SET = utf8mb4 COLLATE = utf8mb4_general_ci ROW_FORMAT = Dynamic;
                """;
            MySqlConnection conn = GetAndOpenDBConn("iFileProxy_Db");

            try
            {
                using MySqlCommand cmd = new(sql, conn);
                cmd.ExecuteNonQuery();
            }
            catch (Exception)
            {
                _logger.Error($"Db Init Fai.l");
                throw;
            }
            finally
            {
                conn.Close();
            }
        }

        /// <summary>
        /// 执行标量查询
        /// </summary>
        /// <typeparam name="T">返回值类型</typeparam>
        /// <param name="sql">SQL查询语句</param>
        /// <param name="parameters">查询参数</param>
        /// <returns>查询结果</returns>
        public T ExecuteScalar<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return (T)cmd.ExecuteScalar();
        }

        /// <summary>
        /// 执行查询并返回实体列表
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="sql">SQL查询语句</param>
        /// <param name="parameters">查询参数</param>
        /// <returns>实体列表</returns>
        public List<T> ExecuteQuery<T>(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return JsonConvert.DeserializeObject<List<T>>(QueryTableData(cmd));
        }

        /// <summary>
        /// 执行非查询SQL命令
        /// </summary>
        /// <param name="sql">SQL命令</param>
        /// <param name="parameters">命令参数</param>
        /// <returns>受影响的行数</returns>
        public int ExecuteNonQuery(string sql, Dictionary<string, object> parameters)
        {
            using var conn = GetAndOpenDBConn("iFileProxy_Db");
            using var cmd = new MySqlCommand(sql, conn);
            foreach (var param in parameters)
            {
                cmd.Parameters.AddWithValue(param.Key, param.Value);
            }
            return cmd.ExecuteNonQuery();
        }
    }
}
